<?php

declare(strict_types=1);

namespace mirzaev\ebala\controllers;

// Файлы проекта
use mirzaev\ebala\controllers\core,
	mirzaev\ebala\models\settings as model,
	mirzaev\ebala\models\core as _core;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// System libraries
use datetime,
	datetimezone,
	exception;

/**
 * Контроллер настроек сайта
 *
 * @package mirzaev\ebala\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class settings extends core
{
	/**
	 * Страница настроек
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function index(array $parameters = []): ?string
	{
		if ($this->account->status() && $this->account->type === 'administrator') {
			// Авторизован аккаунт (администратор)

			// Чтение настроек
			$this->view->settings = model::search(
				before: 'FILTER setting.category != null'
			);

			// Генерация представления
			$main = $this->view->render(DIRECTORY_SEPARATOR . 'pages' . DIRECTORY_SEPARATOR . 'settings.html');

			// Возврат (успех)
			if ($_SERVER['REQUEST_METHOD'] === 'GET') return $this->view->render(DIRECTORY_SEPARATOR . 'index.html', ['main' => $main]);
			else if ($_SERVER['REQUEST_METHOD'] === 'POST') return $main;
		}

		// Возврат (провал)
		return null;
	}

	/**
	 * Записать
	 *
	 * Записывает (обновляет) в ArangoDB 
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void
	 */
	public function write(array $parameters = []): void
	{
		try {
			if ($this->account->status() && $this->account->type === 'administrator') {
				// Авторизован аккаунт администратора

				// Инициализация инстанции настройки
				$setting = model::read('d._key == "' . $parameters['id'] . '"');

				if ($setting instanceof _document) {
					// Найдена инстанция настройки

					// Запись значения
					$setting->value = $parameters['value'];

					if (_core::update($setting)) {
						// Записано в ArangoDB
					} else throw new exception('Не удалось обновить заявку');
				} else throw new exception('Не найдена заявка');
			} else throw new exception('Вы не авторизованы');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$this->errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}
	}
}
